/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * pathport.h
 * Interface to libpathport
 * Copyright (C) 2006 Simon Newton
 */

#ifndef _defined_pathport_h
#define _defined_pathport_h

#include <stdint.h>
// order is important here for osx
#include <sys/types.h>
#include <sys/select.h>
#include <pathport/common.h>

#ifdef __cplusplus
extern "C" {
#endif

// error codes
enum {
  PATHPORT_EOK = 0,
  PATHPORT_ENET = -1,    // network error
  PATHPORT_EMEM = -2,    // memory error
  PATHPORT_EARG = -3,    // argument error
  PATHPORT_ESTATE =-4,  // state error
  PATHPORT_EACTION = -5  // invalid action
};

static const int PathportMaxUni = 63;  // max universes index


/*
 * Pathport manufactures
 */
enum pathport_manuf_e {
  PATHPORT_MANUF_PATHWAY_CONNECTIVITY = 0x00,
  PATHPORT_MANUF_INTERACTIVE_TECH = 0x10,
  PATHPORT_MANUF_ENTERTAINMENT_TECH = 0x11,
  PATHPORT_MANUF_MA_LIGHTING = 0x12,
  PATHPORT_MANUF_HIGH_END = 0x13,
  PATHPORT_MANUF_CRESTRON = 0x14,
  PATHPORT_MANUF_LEVITON = 0x15,
  PATHPORT_MANUF_FLYING_PIG = 0x16,
  PATHPORT_MANUF_HORIZON = 0x17,
  PATHPORT_MANUF_ZP_TECH = 0x28,

  PATHPORT_MANUF_JANDS = 0xf0
};
typedef enum pathport_manuf_e pathport_manuf;


/*
 * Pathport device class definitions
 */
enum pathport_class_e {
  PATHPORT_CLASS_NODE = 0x00,
  PATHPORT_CLASS_MANAGER = 0x01,
  PATHPORT_CLASS_DIMMER = 0x02,
  PATHPORT_CLASS_CONTROLLER = 0x03,
  PATHPORT_CLASS_FIXTURE = 0x04,
  PATHPORT_CLASS_EFFECTS = 0x05,
  PATHPORT_CLASS_MAX
};
typedef enum pathport_class_e pathport_class;


/*
 * Class types
 */
enum pathport_class_type_e {
  PATHPORT_CLASS_NODE_PATHPORT = 0x00,
  PATHPORT_CLASS_NODE_DMX_MANAGER_PLUS = 0x01,
  PATHPORT_CLASS_NODE_ONEPORT = 0x02,
  PATHPORT_CLASS_MANAGER_PATHPORT = 0x00,
  PATHPORT_CLASS_DIMMER_RACK = 0x00,
  PATHPORT_CLASS_DIMMER_PACK = 0x00,
  PATHPORT_CLASS_CONTROLLER_INTENSITY = 0x00,
  PATHPORT_CLASS_CONTROLLER_MOVING = 0x01,
  PATHPORT_CLASS_FIXTURE_PROJECTOR = 0x00,
  PATHPORT_CLASS_FIXTURE_YOKE = 0x01,
  PATHPORT_CLASS_FIXTURE_MIRROR = 0x01,
  PATHPORT_CLASS_EFFECTS_COLOR = 0x00,
  PATHPORT_CLASS_EFFECTS_MECHINAICAL = 0x01,
  PATHPORT_CLASS_EFFECTS_MOTION = 0x02,
  PATHPORT_CLASS_EFFECTS_ATMOSPHERIC = 0x03,
  PATHPORT_CLASS_TYPE_MAX
};
typedef enum pathport_class_type_e pathport_class_type;


enum pathport_param_e {
  PATHPORT_PARAM_NODE_NAME = 0x0401,
  PATHPORT_PARAM_MAC_ADDR = 0x0421,
  PATHPORT_PARAM_IP_ADDR = 0x0422,
  PATHPORT_PARAM_SUBNET_MASK = 0x0423,
  PATHPORT_PARAM_GATEWAY = 0x0424,
  PATHPORT_PARAM_PATHPORT_ID = 0x0461,
  PATHPORT_PARAM_BACKLIGHT = 0x0481,
  PATHPORT_PARAM_SW_VERSION = 0x0482,
  PATHPORT_PARAM_HW_TYPE = 0x0483,
  PATHPORT_PARAM_LDR_VERSION = 0x0484,
  PATHPORT_PARAM_IDENTIFY = 0x0485,
  PATHPORT_PARAM_IR_ENABLE = 0x0486,
  PATHPORT_PARAM_SERIAL = 0x0487,
  PATHPORT_PARAM_BOOT_ORDER = 0x04a2,
  PATHPORT_PARAM_TERMINAL_CONNECTORS = 0x04c2,
  PATHPORT_PARAM_RACK_MOUNT = 0x04c3,
  PATHPORT_PARAM_BOX_STYLE = 0x04cc,

  PATHPORT_PARAM_REBOOT = 0x04a1,
  PATHPORT_PARAM_COMPONENT_LABEL = 0x0411,
  PATHPORT_PARAM_PATCH_NAME = 0x0412,
  PATHPORT_PARAM_DMX_SPEED = 0x0413,
  PATHPORT_PARAM_IS_BIDIRECTIONAL = 0x0414,
  PATHPORT_PARAM_IS_PHYSICAL = 0x0415,
  PATHPORT_PARAM_IS_MALE = 0x0416,
  PATHPORT_PARAM_IS_SINK = 0x0417,
  PATHPORT_PARAM_XDMX_COUNT = 0x0418,
  PATHPORT_PARAM_ALT_PATCH_MODE_ENABLE = 0x0419,
  PATHPORT_PARAM_ALT_START_CODE = 0x041A,
  PATHPORT_PARAM_MAX_PATCHES = 0x041B,
  PATHPORT_PARAM_NUM_PATCHES = 0x041C,
  PATHPORT_PARAM_QUICK_PATCH_UNI = 0x041D,
  PATHPORT_PARAM_XDMX_SRC_OFFSET = 0x0463,


  PATHPORT_PARAM_PORT_ENABLE = 0x04c4,
  PATHPORT_PARAM_DMX_ACTIVE = 0x04c5,
  PATHPORT_PARAM_XDMX_ACTIVE = 0x04c6,
  PATHPORT_PARAM_SIG_LOSS_HOLD = 0x04c7,
  PATHPORT_PARAM_SIG_LOSS_HOLD_FOREVER = 0x04c8,
  PATHPORT_PARAM_SIG_LOSS_FADE_ENABLE = 0x04c9,
  PATHPORT_PARAM_SIG_LOSS_FADE_TIME = 0x04ca,
  PATHPORT_PARAM_SIG_LOSS_PORT_SHUTDOWN = 0x04cb,



  PATHPORT_PARAM_TALKBACK = 0x0605,

}__attribute__( ( packed ) );
typedef enum pathport_param_e pathport_param;


/*
 * port directions
enum pathport_port_mode_e {
  PATHPORT_PORT_MODE_DISABLED,
  PATHPORT_PORT_MODE_OUT,
  PATHPORT_PORT_MODE_IN,
  PATHPORT_PORT_MODE_MOUT,
  PATHPORT_PORT_MODE_MIN
};
typedef enum pathport_port_mode_e pathport_port_mode_t;
*/


/*
 * Enums for the application defined handlers
 */
typedef enum {
  PATHPORT_RECV_HANDLER,
  PATHPORT_ARPREQ_HANDLER,
  PATHPORT_ARPREP_HANDLER,
  PATHPORT_DMX_HANDLER,
  PATHPORT_LAST_HANDLER
} pathport_handler_name_t;


enum { PATHPORT_MAX_SD = 4 };

enum { PATHPORT_MAX_UNIVERSES = 64 };

/** The pathport node */
typedef void *pathport_node;

extern char *pathport_strerror();

// node control functions
extern pathport_node pathport_new(const char *ip, int verbose);
extern int pathport_start(pathport_node n);
extern int pathport_read(pathport_node n, int timeout);
extern int pathport_stop(pathport_node n);
int pathport_destroy(pathport_node vn);

// handler functions
extern int pathport_set_dmx_handler(pathport_node vn, int (*fh)(pathport_node n, unsigned int universe, unsigned int length, const uint8_t *data, void *d), void *data);
extern int pathport_set_handler(pathport_node vn, pathport_handler_name_t handler, int (*fh)(pathport_node n, void *pp, void * d), void* data);

// send functions
extern int pathport_send_reply(pathport_node vn);
extern int pathport_send_dmx(pathport_node n, unsigned int uni, uint16_t length, const uint8_t *data);
extern int pathport_release_dmx(pathport_node n, unsigned int uni);

// config functions
extern int pathport_set_name(pathport_node vn, const char *name);
extern int pathport_set_type(pathport_node vn, pathport_manuf manu, pathport_class cls, pathport_class_type type);
extern int pathport_set_id(pathport_node vn, uint32_t id);
extern int pathport_register_uni(pathport_node vn, uint8_t uni);
extern int pathport_unregister_uni(pathport_node vn, uint8_t uni);

// misc
extern int pathport_get_sd(pathport_node n, unsigned int i);

#ifdef __cplusplus
}
#endif

#endif
